
log.dens.target=function(theta=c(1:2), target.params=1)
{
rez= dmvnorm(theta, c(0,0), diag(1,2), log=T)
return(rez)
}


ACMTM_sampling = function(log.dens.target,N.iter,theta.init,sig.mat,alpha=2.9,adp=TRUE,vec=FALSE,print.out.freq=N.iter+1,...) 
{
  L.x=10^100       # Defines a compact set K. The absolute value for each coordinate sampled must be less than L.x 
  L.sig=10^100     # Upper bound for all sigma proposals
  eps.sig=10^-100  # Lower bound for all sigma proposals
  propburn=100     # interval of iterations when evaluating if adaption is necessary 
  odds =1          # chance of adaption happening if adaption condition (rule) is satisfied
  restart=1        # resets to the iteration number where the last adaption occurred
  adj=F            # turns T if adaption occurs, and turns F again after that iteration.

  p = length(theta.init)
  if( is.vector(sig.mat) )
    sig.mat = matrix(rep(sig.mat,p),ncol=p)

  nprop = dim(sig.mat)[1] 
  maxiter = N.iter*p
  percentmax=2*1/nprop 
  percentmin=0.5*1/nprop

  if( (max(sig.mat)>L.sig) || (min(sig.mat)<eps.sig) )
    return( cat("Some sigma proposals are very large or very small! \n") )
   
  if( max(abs(theta.init)) > L.x )
    return( cat("Some initial parameters are very large or very small! \n") )

  #----------------------------------------------------
  Xveclist = matrix(NA,maxiter,p)
  accept   = matrix(0,N.iter,p)
  select   = matrix(NA,N.iter,p)

  X = theta.init
  gX = log.dens.target(X,...)

  for(i in 1:N.iter)
  {
     if(adp == TRUE)
     {
      # checking if adaption is necessary or not at every predetermined point
      if ((i>propburn)&((i-1)%%propburn==0)){
        if (runif(1)<=odds){  
          for (coord in 1:p) {
            temp=c(sum(select[(restart:(i-1)),coord]==1), sum(select[(restart:(i-1)),coord]==nprop))
            if ( (temp[2]>((i-restart)*percentmax)) & (2*sig.mat[nprop,coord]<L.sig) ){
              for (j in 2:nprop){sig.mat[j,coord]=sig.mat[1,coord]*((2*sig.mat[nprop,coord])/sig.mat[1,coord])^((j-1)/(nprop-1))}
              adj=T
            } 
            else if ((temp[2]<((i-restart)*percentmin)) & (sig.mat[1,coord]<0.5*sig.mat[nprop,coord]) ){
              for (j in 2:nprop){sig.mat[j,coord]=sig.mat[1,coord]*((0.5*sig.mat[nprop,coord])/sig.mat[1,coord])^((j-1)/(nprop-1))}
              adj=T
            } 
            
            if ( (temp[1]>((i-restart)*percentmax)) & (0.5*sig.mat[1,coord]>eps.sig) ){
              sig.mat[1,coord]=sig.mat[1,coord]/2
              for (j in 2:(nprop-1)){sig.mat[j,coord]=sig.mat[1,coord]*(sig.mat[nprop,coord]/sig.mat[1,coord])^((j-1)/(nprop-1))}
              adj=T
            }
            else if ((temp[1]<((i-restart)*percentmin)) & (sig.mat[1,coord]*2<sig.mat[nprop,coord]) ){
              sig.mat[1,coord]=sig.mat[1,coord]*2
              for (j in 2:(nprop-1)){sig.mat[j,coord]=sig.mat[1,coord]*(sig.mat[nprop,coord]/sig.mat[1,coord])^((j-1)/(nprop-1))}
              adj=T
            }

          }
          if (adj==T){restart=i; adj=F}   
        }
        a.temp = floor( (i-1)/propburn )   # Number of adaptation points
        odds = max( c( 0.99^(a.temp-1), 1/sqrt(a.temp) ) )
      }
    }  # end of if(adp==TRUE) 
      

    Y=Xstar=matrix(rep(X,nprop),p,nprop)
    for (coord in 1:p) { 
        iter = (i-1)*p + coord
        
        # propose possible Y values
        Y[coord,] = X[coord] + rnorm(nprop,sd = sig.mat[,coord])  
        if(vec==TRUE) { gY=log.dens.target(t(Y),...) }
        else          { gY=apply(Y,2,function(x){log.dens.target(x,...)}) }
        
        if ( (max(gY)==-Inf)||(sum(abs(Y)>L.x)>0) )    {
          Y[coord,] = rep(X[coord],nprop)
          Xveclist[iter,] = X
        } 
        else {
	  # select Y among `nprop' proposal
          dist1 = Y[coord,]-X[coord]
          logW_Y=gY+alpha/2*log(dist1^2)                    
          tempadj = max(logW_Y)
          cumW = cumsum(exp(logW_Y-tempadj))
          U = runif(1)
          U=U*(cumW[nprop])
          sel = which(cumW>U)[1]
          select[i,coord]=sel
          Yselect = Y[,sel]
          
          # propose possible X* values
          Xstar[coord,] = Yselect[coord] + rnorm(nprop,sd = sig.mat[,coord])    
          Xstar[coord,sel]=X[coord]
          
          # compute probability for each X* 
          if(vec==TRUE) { gXstar=log.dens.target(t(Xstar),...) }
          else          { gXstar=apply(Xstar,2,function(x){log.dens.target(x,...)}) }

          dist2 = Xstar[coord,]-Yselect[coord]
          logW_Xstar=gXstar+alpha/2*log(dist2^2)        
          
          # accept/reject
          if (exp(max(logW_Xstar))==0){
            tempadj = max(logW_Xstar)
            rho = min (1, (sum(exp(logW_Y-tempadj))/sum(exp(logW_Xstar-tempadj))))
          }
          else{
            rho = min (1, (sum(exp(logW_Y))/sum(exp(logW_Xstar))))
          }  
           
          if (runif(1) < rho) {
            gX <- gY[sel]
            X[coord]<-Yselect[coord]
            accept[i,coord] = 1
          } # accept proposal
          Y[coord,]      <- rep(X[coord],nprop)
          Xstar[coord,]  <- rep(X[coord],nprop)
          Xveclist[iter,] = X
        }
      } # end for(coord) loop

      if( (i%%print.out.freq)==0 )
        cat("Just passed ",i," iteration.\n")
    
   } # end of for(i) loop

   Xveclist=Xveclist[seq(1,maxiter,by=p),]

   return(list(samples=Xveclist,sig.mat=sig.mat,select=select,accept=accept,last.adp=restart))
}
